import { Injectable } from '@angular/core';
import { Post } from '../../../model/Post';
import { AuthService } from '../auth/auth.service';
import { AngularFirestore, Reference } from '@angular/fire/compat/firestore';
import { User } from '../../../model/User';
import { UserService } from './user.service';
import { firstValueFrom } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class PostService {
  constructor(private auth: AuthService, private afs: AngularFirestore, private usersvc: UserService) { }

  getFeed(length: Number, start: Number): Promise<Array<Post>> {
    return new Promise<Array<Post>>(async (resolve, reject) => {
      if (!this.auth.checkAllowed())
        reject();

      let result: Array<Post> = new Array<Post>();
      let currentUser = await this.usersvc.getCurrentUser();

      let posts = await firstValueFrom(this.afs.collection('post', ref => ref.orderBy('date', 'desc')).valueChanges({ idField: 'id' }));
      posts.forEach(async post=>{
        let user = await firstValueFrom(this.afs.doc((post as any).user).valueChanges()) as User;

        let likes = await firstValueFrom(this.afs.collection('like', ref => ref.where('post', '==', this.afs.doc('/post/' + post.id).ref)).valueChanges())

        result.push({
          user: user,
          content: (post as any).content,
          image: (post as any).image,
          likes: likes.length,
          liked: likes.filter(like => (like as any).user.id == currentUser.username).length != 0,
          id: post.id
        })
        resolve(result);
      })
    })
  }

  getPosts(username: String | undefined): Promise<Array<Post>> {
    return new Promise<Array<Post>>(async (resolve, reject) => {

      if (!this.auth.checkAllowed())
        reject()

      let result: Array<Post> = new Array<Post>;

      if (username == undefined) {
        // /user -> currentUser
        let user = await this.usersvc.getCurrentUser();
        let userRef = this.afs.doc('/user/' + user.username).ref;
        let posts = (await firstValueFrom(this.afs.collection<Post>('post', ref => ref.where('user', '==', userRef).orderBy('date', 'desc')).valueChanges({ idField: 'id' })));
        posts.forEach(async (post) => {
          let likes = await firstValueFrom(this.afs.collection('like', ref => ref.where('post', '==', this.afs.doc('/post/' + post.id).ref)).valueChanges());
          result.push({
            user: user,
            content: post.content,
            image: post.image,
            likes: likes.length,
            liked: likes.filter(like => (like as any).user.id == user.username).length != 0,
            id: post.id
          })

        })
        resolve(result);
      } else {
        // /user/name
        let user = await this.usersvc.getUser(username);
        let userRef = this.afs.doc('/user/' + user.username).ref;
        let currentUser = await this.usersvc.getCurrentUser();
        let posts = (await firstValueFrom(this.afs.collection<Post>('post', ref => ref.where('user', '==', userRef).orderBy('date', 'desc')).valueChanges({ idField: 'id' })));
        posts.forEach(async (post) => {
          let likes = await firstValueFrom(this.afs.collection('like', ref => ref.where('post', '==', this.afs.doc('/post/' + post.id).ref)).valueChanges());
          result.push({
            user: user,
            content: post.content,
            image: post.image,
            likes: likes.length,
            liked: likes.filter(like => (like as any).user.id == currentUser.username).length != 0,
            id: post.id
          })
        })
        resolve(result);
      }
    });
  }

  newPost(content: String, image: String): Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      if (!this.auth.checkAllowed())
        reject();

      let user = await this.usersvc.getCurrentUser();
      this.afs.collection('post').add({
        user: this.afs.doc('/user/' + user.username).ref,
        content: content,
        image: image,
        date: Date.now()
      })
        .then(() => resolve())
        .catch(() => reject())
    });
  }

  removePost(id: String) {
    return this.afs.doc('/post/' + id).delete();
  }

  toggleLike(postId: String): Promise<boolean> {
    return new Promise<boolean>(async (resolve, reject) => {
      if (!this.auth.checkAllowed())
        reject();

      let user = await this.usersvc.getCurrentUser();
      let userRef = this.afs.doc('/user/' + user.username).ref;
      let postRef = this.afs.doc('/post/' + postId).ref;

      firstValueFrom(this.afs.collection('like', ref => ref.where('user', '==', userRef).where('post', '==', postRef)).valueChanges({ idField: 'id' }))
        .then(async likes => {
          if (likes.length == 0) {
            await this.afs.collection('like').add({
              user: userRef,
              post: postRef
            })
            resolve(true);
          } else {
            likes.forEach(async like => {
              await this.afs.doc('/like/' + like.id).delete();
              resolve(false);
            })
          }
          console.log(likes)
        })
    })
  }
}
